#include "CmdAutoDrive.h"
#include "Robot.h"

CmdAutoDrive::CmdAutoDrive(double distance, bool useAbsolute, double maxDrive, int msecTimeout, bool useVision,
		                   bool ignoreTimeout, bool* cancel) {
	Requires(drive.get());

	m_distance = distance;
	m_useAbsolute = useAbsolute;
	m_maxDrive = maxDrive;
	m_msecTimeout = msecTimeout;
	m_useVision = useVision;
	m_ignoreTimeout = ignoreTimeout;
	m_cancel = cancel;
}

void CmdAutoDrive::Initialize() {
	if(m_cancel != nullptr && *m_cancel == true) return;

	drive->InitAutoDrive(m_distance, m_useAbsolute, m_maxDrive, m_msecTimeout, m_useVision);
}

void CmdAutoDrive::Execute() {
}

bool CmdAutoDrive::IsFinished() {
	bool isFinished = false;

	switch(drive->GetAutoState()){
		case Drive::asFinished:
			isFinished = true;
			break;
		case Drive::asTimedOut:
			if(!m_ignoreTimeout) *m_cancel = true;
			isFinished = true;
			break;
		default:;
	}

	return isFinished;
}

void CmdAutoDrive::End() {
	drive->SetDriveMode(Drive::dmAutoDone);
}

void CmdAutoDrive::Interrupted() {
	drive->SetDriveMode(Drive::dmDriveRotate);
	Robot::robotLog->Write("Drive:   Auto Drive Interrupted");
}
