#include "PeriodicRobot.h"

#include "DriverStation.h"
#include "HAL/HAL.h"
#include "LiveWindow/LiveWindow.h"

// ************************ PROTECTED FUNCTIONS ***********************

PeriodicRobot::PeriodicRobot()
{
	m_mode = mInit;
	m_period = 0.02;
	m_notifier = new Notifier(PeriodicRobot::CallPeriodic, this);
}

PeriodicRobot::~PeriodicRobot()
{
	delete m_notifier;
}

void PeriodicRobot::Periodic()
{
	if (IsDisabled()) {
		if (m_mode != mDisabled) {
			m_mode = mDisabled;
			DisabledInit();
            Scheduler::GetInstance()->SetEnabled(false);
		}

		HAL_ObserveUserProgramDisabled();
		DisabledPeriodic();

	} else if (IsAutonomous()) {
		if (m_mode != mAuto) {
			m_mode = mAuto;
			AutonomousInit();
            Scheduler::GetInstance()->SetEnabled(true);
		}

		HAL_ObserveUserProgramAutonomous();
		AutonomousPeriodic();

	} else if (IsTest()) {
		if (m_mode != mTest) {
			m_mode = mTest;
			TestInit();
            Scheduler::GetInstance()->SetEnabled(true);
		}

		HAL_ObserveUserProgramTest();
		TestPeriodic();

	} else {
		if (m_mode != mTeleop) {
			m_mode = mTeleop;
			TeleopInit();
            Scheduler::GetInstance()->SetEnabled(true);
		}

		HAL_ObserveUserProgramTeleop();
		TeleopPeriodic();
	}
}

void PeriodicRobot::Prestart()
{
}

// ************************* PUBLIC FUNCTIONS *************************

void PeriodicRobot::RobotInit()
{
}

void PeriodicRobot::DisabledInit()
{
}

void PeriodicRobot::AutonomousInit()
{
}

void PeriodicRobot::TeleopInit()
{
}

void PeriodicRobot::TestInit()
{
}

void PeriodicRobot::DisabledPeriodic()
{
}

void PeriodicRobot::AutonomousPeriodic()
{
}

void PeriodicRobot::TeleopPeriodic()
{
}

void PeriodicRobot::TestPeriodic()
{
}

void PeriodicRobot::SetPeriod(double period)
{
	if (period <= 0) period = 0.02;
	m_period = period;
}

void PeriodicRobot::StartCompetition()
{
	HAL_Report(HALUsageReporting::kResourceType_Framework,
	           HALUsageReporting::kFramework_Iterative);

	SmartDashboard::init();
	NetworkTable::GetTable("LiveWindow")->GetSubTable("~STATUS~")->PutBoolean("LW Enabled", false);
	LiveWindow *lw = LiveWindow::GetInstance();

	RobotInit();
	m_notifier->StartPeriodic(m_period);

	HAL_ObserveUserProgramStarting();
	lw->SetEnabled(false);

	while(true) Wait(1.0);
}

// ************************* PRIVATE FUNCTIONS ************************

void PeriodicRobot::CallPeriodic(void *periodicRobot)
{
	PeriodicRobot* robot = (PeriodicRobot*) periodicRobot;
	robot->Periodic();
}

