#ifndef ROBOT_H_
#define ROBOT_H_

#include "WPILib.h"
#include <cstdlib>
#include "PeriodicRobot.h"
#include "RobotLog.h"
#include "TcpHost.h"
#include "PIDControl.h"

class Robot: public PeriodicRobot {
public:
	static std::unique_ptr<RobotLog> robotLog;
	static std::unique_ptr<Dashboard> dashboard;
//	static double* dashValue;

	// RoboRIO Channels

	enum CanId {canLowShooter = 1, canHighShooter};

	enum DIOChannel {dioDriveEncoderRA, dioDriveEncoderRB, dioDriveEncoderLA, dioDriveEncoderLB, dioGearSensor};

	enum PwmChannel {pwmDriveMotorLF, pwmDriveMotorLR, pwmDriveMotorRF, pwmDriveMotorRR, pwmFuelIntake,  pwmGoator,
				     pwmGearRollerL, pwmGearRollerR, pwmFuelChurner};

    enum RelayChannel {relFuelAgitator, relFuelAuger};

    enum SolenoidChannel {solGearArm,solRightLeg, solLeftLeg, solAirVent};

    // Dashboard Data Indexes

    enum DashButton {dbGyroCalibrate, dbRunPid, dbLogData};

    enum DashValue {dvAutoSelect, dvAutoDelay, dvDutoStart, dvPidSelect, dvPidSetpoint, dvPidMaxOut, dvPthreshold,
	                dvPabove, dvPbelow, dvIthreshold, dvIabove, dvIbelow, dvDthreshold, dvDabove,
	                dvDbelow, dvMinFuelIntake, dvMaxFuelIntake, dvMaxDriveSpeed, dvLowShooterSpeed,
					dvHighShooterSpeed, dvGearInSpeed, dvGearOutSpeed, dvGoatDirection, dvFuelChurnerSpeed};

	enum RobotValue {rvGyroCal, rvDriveGyro, rvDriveEncoderL, rvDriveEncoderR, rvDriveMode, rvFuelIntake, rvFuelLegs,
					 rvLowShooter, rvHighShooter,  rvGearSensor, rvLiftAlign, rvGoatAmps, rvAirVent};

	void StartCamera();


private:
	void RobotInit();
	void DisabledInit();
	void DisabledPeriodic();
	void AutonomousInit();
	void AutonomousPeriodic();
	void TeleopInit();
	void TeleopPeriodic();
	void TestInit();
	void TestPeriodic();

	char m_log[100];

	CommandGroup*		m_autoCommand;
	PIDControl*      	m_tunePID;
	Solenoid*			m_cameraLED;

	double				m_autoDelay;
	int 				m_autoSelect;

	cs::UsbCamera 		m_aimCamera;
};

#endif
