#ifndef Drive_H
#define Drive_H

#include <Commands/Subsystem.h>
#include "WPILib.h"
#include "PIDControl.h"

class Drive : public Subsystem {

public:
	typedef enum {dwAll, dwRear, dwFront, dwPivotLeft, dwPivotRight} DriveWheels;
	typedef enum {dmDriveRotate, dmDriveStrafe, dmStrafe, dmJoystick,
				  dmAutoRotate, dmAutoDrive, dmAutoStrafe, dmAutoDone, dmGearPlace} DriveMode;
	typedef enum {asNull, asInProgress, asFinished, asTimedOut} AutoState;

	Drive();

	void 	  	CalibrateGyro();
	double    	ErrorDrive(double target);
	double 		ErrorRotate(double target);
	AutoState 	GetAutoState();
	double		GetDriveDistance();
	double 	  	GetDriveRate();
	double 	 	GetGyroAngle();

	void		InitAutoDrive(double distance, bool useAbsolute, double maxDrive, int msecTimeout, bool useVision = false);
	void 		InitAutoRotate(double angle, bool useAbsolute, double maxRotate, int msecTimeout, DriveWheels wheels);
	void		InitAutoStrafe(double distance, bool useAbsolute, double maxStrafe, int msecTimeout);

	void 		InitDefaultCommand();
	void 		InitDrive();
	void		InitRotatePID(double addAngle = 0);
	void 		Periodic();
	void		ResetNavigation();
	void		SetDashboardValues();
	void 		SetDriveMode(DriveMode mode, int gearPlaceTimeout = 0);
	void    	TuneDrivePID(double pidOutput);
	void 		TuneRotatePID(double pidOutput);
	void 		TuneStrafePID(double pidOutput);

private:
	VictorSP*		m_motorLF;
	VictorSP*		m_motorLR;
	VictorSP*		m_motorRF;
	VictorSP*		m_motorRR;

	ADXRS450_Gyro*	m_gyro;
	Encoder* 		m_driveEncoderL;
	Encoder*        m_driveEncoderR;

	PIDControl*		m_rotatePID;
	PIDControl*		m_drivePID;

	AutoState		m_autoState;
	DriveMode		m_driveMode;
	char			m_log[100];
	double 			m_targetAngle;
	double			m_targetDrive;
	DriveWheels		m_driveWheels;
	bool			m_useVision;

	std::string	DriveModeName(DriveMode mode);
	double 		Limit(double Value);
	void 		MecanumDrive(double drive, double strafe, double rotate, DriveWheels drivewheels);

	int 			m_timeoutCounter;
};

#endif  // Drive_H
