#include "GearPickup.h"
#include "../Robot.h"
#include "../CommandBase.h"

GearPickup::GearPickup() : Subsystem("GearPickup") {
	m_airVent 		= nullptr;
	m_arm			= nullptr;
	m_compressor	= nullptr;
	m_gearSensor	= nullptr;
	m_rollerL		= nullptr;
	m_rollerR		= nullptr;

	m_ejectCount = 0;
	m_gearDown 	 = false;
	m_rollerMode = rmOff;
	m_rollerPWM  = 0;
	m_ventOpen	 = false;
}

bool GearPickup::GetGearPosition() {
	return m_gearDown;
}

void GearPickup::InitDefaultCommand() {
}

void GearPickup::InitGearPickup() {
	m_airVent	  = new Solenoid(Robot::solAirVent);
	m_arm    	  = new Solenoid(Robot::solGearArm);
	m_compressor  = new Compressor();
	m_gearSensor  = new DigitalInput(Robot::dioGearSensor);
	m_rollerL 	  = new VictorSP(Robot::pwmGearRollerL);
	m_rollerR 	  = new VictorSP(Robot::pwmGearRollerR);
}

void GearPickup::Periodic() {
	double pwmLeft = m_rollerPWM;
	double pwmRight = -m_rollerPWM;

	if(m_rollerMode == rmIn) {
		if(!m_gearSensor->Get()) {
			SetRollerMode(rmOff);
			SetGearPosition(false);
			Robot::dashboard->SetRobotValue(Robot::rvGearSensor, 1);
		}

	} else if(m_rollerMode == rmPlace) {
		m_ejectCount--;
		if(m_ejectCount <= 0) {
			SetRollerMode(rmOff);
			Robot::dashboard->SetRobotValue(Robot::rvGearSensor, 0);
		}

	} else if(m_rollerMode == rmEject){
		m_ejectCount--;
		if(m_ejectCount <= 0) SetRollerMode(rmEjectWait);

	} else if(m_rollerMode == rmEjectWait){
		m_ejectCount--;
		if(m_ejectCount <= 0) SetRollerMode(rmIn);

	} else if(m_rollerMode == rmRollGear) {
		m_ejectCount--;
		if(m_ejectCount <= 0) {
			SetRollerMode(rmOff);
		} else {
			pwmLeft = 0.2;
			pwmRight = 0.35;
		}
	}

	m_rollerL->Set(pwmLeft);
	m_rollerR->Set(pwmRight);
//	m_rollerL->Set(m_rollerPWM);
//	m_rollerR->Set(-m_rollerPWM);
}

void GearPickup::SetAirVent(bool ventOpen) {
	if (m_ventOpen != ventOpen) {
		m_ventOpen = ventOpen;
		m_airVent->Set(m_ventOpen);

		if (m_ventOpen) {
			m_compressor->Stop();
			Robot::dashboard->SetRobotValue(Robot::rvAirVent, 1);
		} else {
			m_compressor->Start();
			Robot::dashboard->SetRobotValue(Robot::rvAirVent, 0);
		}
	}
}

std::string GearPickup::RollerModeName(int index) {
	switch (index) {
		case 0: 	return "Off";
		case 1: 	return "In";
		case 2: 	return "Out";
		case 3: 	return "Place";
		case 4:		return "Eject";
		case 5:		return "Eject Wait";
		case 6:		return "Roll Gear";
		default:	return "?";
	}
}

void GearPickup::SetDashboardValues() {
	if(m_gearSensor->Get()) Robot::dashboard->SetRobotValue(Robot::rvGearSensor, 0);
	else Robot::dashboard->SetRobotValue(Robot::rvGearSensor, 1);
}

void GearPickup::SetGearPosition(bool gearDown) {
	if (m_gearDown != gearDown){
		m_arm->Set(gearDown);
		m_gearDown = gearDown;

		if (m_gearDown) Robot::robotLog->Write("Gear:    Gear Arm=Down");
		else			Robot::robotLog->Write("Gear:    Gear Arm=Up");
	}
}

void GearPickup::SetRollerMode(RollerMode mode) {
	switch(mode) {
		case rmOff:
			m_rollerPWM = 0;
			break;

		case rmIn:
			if(m_gearSensor->Get()) {
				m_rollerPWM = Robot::dashboard->GetDashValue(Robot::dvGearInSpeed);
				Robot::dashboard->SetRobotValue(Robot::rvGearSensor, 0);
			} else {
				m_rollerPWM = 0;
				mode = rmOff;
			}
			break;

		case rmOut:
			m_rollerPWM = Robot::dashboard->GetDashValue(Robot::dvGearOutSpeed);
			break;

		case rmPlace:
			m_ejectCount = 100;
			m_rollerPWM = Robot::dashboard->GetDashValue(Robot::dvGearOutSpeed);
			break;

		case rmEject:
			m_ejectCount = 50;
			SetGearPosition(true);
			break;

		case rmEjectWait:
			m_ejectCount = 30;
			m_rollerPWM = Robot::dashboard->GetDashValue(Robot::dvGearOutSpeed);
			break;

		case rmRollGear:
			m_ejectCount = 18;
			//m_rollerPWM = 0.2;
			break;
		default:;
	}

	if (m_rollerMode != mode) {
		m_rollerMode = mode;
		sprintf(m_log, "Gear:    Roller Mode=%s", RollerModeName(mode).c_str());
		Robot::robotLog->Write(m_log);
	}
}
