#ifndef TCPHOST_H_
#define TCPHOST_H_

#include "WPILib.h"
#include "RobotLog.h"

class Dashboard
{
public:
	Dashboard(int robotValueCount, int dashValueCount, int dashButtonCount);
	~Dashboard();

	std::string CountReply();										// Returns current state of Robot, DbData, DbButton values
	std::string DataString(int32_t number, int delimiter);			// Returns string with integer followed by a delimiter
	std::string DataString(double number, int delimiter);			// Returns string with double followed by a delimiter
	bool		GetDashButton(int group, int button);				// Returns state of a Dashboard Button
	bool		GetDashButtonPress(int group, int button);			// Returns whether or not a Dashboard Button has been Pressed
	double		GetDashValue(int index);							// Returns Dashboard value at specified index
	std::string GetReply(int beginIndex, int endIndex);				// Returns GET reply for all Robot Values in the specified range
	std::string GetTimeStamp();

	void        SaveDashValues();
	bool		SetDashButton(int index, int32_t value);			// Sets Dashboard Button group at specified index
	bool		SetDashValue(int index, double value);				// Sets Dashboard Value at specified index
	bool		SetRobotValue(int index, double value);				// Sets Robot Value at specified index
	void		SetRobotMode(int mode);								// Sets Robot Mode
	void        SetTimeStamp(std::string now);
	static void TcpLoop(Dashboard *host);
	void		WriteToLog(std::string entry);

private:
	struct button {
		int32_t state;
		int32_t pressed;
	};

	int			m_dashboardButtonCount;
	int			m_dashboardValueCount;
	int			m_robotValueCount;
	int			m_robotMode;
	std::string m_timeStamp;

	button*		m_dashboardButton;
	double*		m_dashboardValue;
	char		m_log[100];
	double*		m_robotValue;
};

#endif
